<?php

namespace TDevAgency\Cms\Controllers\Admin;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\View;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Cms\Models\BaseModels\Post;
use TDevAgency\Cms\Models\BaseModels\PostLocalization;
use TDevAgency\Cms\Traits\CategoryTrait;
use TDevAgency\Cms\Traits\PostHelper;

class PageController extends Controller
{
    use CategoryTrait, PostHelper;

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function index(Request $request)
    {
        return $this->displayList(Post::PAGE_TYPE, $request, ['index' => 'cms.pages.index', 'edit' => 'cms.pages.edit', 'destroy' => 'cms.pages.destroy']);
    }

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        return View::make('cms::admin.screens.pages.single')->with('post', new Post);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validatorsArray());

        $page = new Post();
        try {
            DB::transaction(function () use ($request, &$page) {
                $this->postPrepare($request, $page);
                $page->post_type = Post::PAGE_TYPE;
                $page->save();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $page);
                }
            });

            return redirect()->route('cms.pages.index')->with('toast', [
                'title' => trans('cms::admin.toast.created'),
                'status' => 'success'
            ]);
        } catch (\Throwable $e) {
            return redirect()->back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_created'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param $id
     */
    public function show($id)
    {
        //
    }

    /**
     * @param Post $page
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function edit($page)
    {
        $page->setRelation('localizations', $page->localizations->mapWithKeys(function (PostLocalization $pageLocalization) {
            return [$pageLocalization->lang => $pageLocalization];
        }));
        return view('cms::admin.screens.pages.single')
            ->with('post', $page);

    }

    /**
     * @param Request $request
     * @param Post $page
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function update(Request $request, $page)
    {
        $this->validate($request, $this->validatorsArray($page->id));

        try {
            DB::transaction(function () use ($request, &$page) {
                $this->postPrepare($request, $page);
                $page->update();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $page);
                }

            });
            return redirect()->route('cms.pages.index')->with('toast', [
                'title' => trans('cms::admin.toast.updated'),
                'status' => 'success'
            ]);
        } catch (\Throwable $e) {
            return redirect()->back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_updated'),
                'status' => 'danger'
            ]);
        }

    }

    /**
     * @param Post $page
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Throwable
     */
    public function destroy($page)
    {
        if ($page->slug !== '/') {
            DB::transaction(function () use ($page) {
                PostLocalization::where('post_id', $page->id)->delete();
                Post::where('id', $page->id)->delete();
            });

            return back()->with('toast', [
                'title' => trans('cms::admin.toast.deleted'),
                'status' => 'success'
            ]);
        }
        return back()->with('toast', [
            'title' => trans('cms::admin.toast.can_not_delete_slash'),
            'status' => 'danger'
        ]);
    }
}
